/***************************************************************************//**
* \file ble_app.c
* \version 1.0
*
* \brief
* Source file for BLE application..
*
********************************************************************************
* \copyright
* Copyright 2016-2018, Cypress Semiconductor Corporation. All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/
#include "ble_common.h"
#include "ble_bond.h"
#include "ble_bas.h"
#include "ble_app.h"
#include "ble_ias.h"

/**
 * @brief Enable or Disable the BLE timer function
 */
#define ENABLE_BLE_MAIN_TIMER                       DISABLED

/**
 * @brief Global Handle to internal BLE structure which holds the BLE connected handle.
 */
cy_stc_ble_conn_handle_t ble_app_conn_handle;

/**
 * @brief Global buffer the battery level.
 */
static uint8_t ble_battery_level = 0;
static bool ble_battery_level_updated_flag = false;

/**
 * @brief Global buffer for generated keys are store.
 */
static cy_stc_ble_gap_sec_key_info_t keyInfo =
{
    .localKeysFlag    = CY_BLE_GAP_SMP_INIT_ENC_KEY_DIST | 
                        CY_BLE_GAP_SMP_INIT_IRK_KEY_DIST | 
                        CY_BLE_GAP_SMP_INIT_CSRK_KEY_DIST,
    .exchangeKeysFlag = CY_BLE_GAP_SMP_INIT_ENC_KEY_DIST | 
                        CY_BLE_GAP_SMP_INIT_IRK_KEY_DIST | 
                        CY_BLE_GAP_SMP_INIT_CSRK_KEY_DIST |
                        CY_BLE_GAP_SMP_RESP_ENC_KEY_DIST |
                        CY_BLE_GAP_SMP_RESP_IRK_KEY_DIST |
                        CY_BLE_GAP_SMP_RESP_CSRK_KEY_DIST,
};

#if ENABLE_FPMM_CMD_RESPONSE_ECHO == ENABLED
/**
 * @brief The flag for BLE FPMM response sent.
 */
static volatile bool fpmm_res_sent_flag = false;

/**
 * @brief The echo data of FPMM response.
 */
#define FPMM_RES_ECHO_DATA_SIZE             2u
static const uint8_t fpmm_res_echo_data[FPMM_RES_ECHO_DATA_SIZE+1] = FPMMCMD_RESPONSE_ECHO_STRING;

/**
 * @brief the BLE response timer parameter.
 */
static cy_stc_ble_timer_info_t  ble_res_timer_param = {.timeout = 1u};

/**
 * @brief the BLE response timer timeout flag.
 */
static volatile bool ble_res_timer_timeout_flag = false;
#endif /* ENABLE_FPMM_CMD_RESPONSE_ECHO == ENABLED */

#if ENABLE_BLE_MAIN_TIMER == ENABLED
static volatile uint32_t        mainTimer  = 1u;
static cy_stc_ble_timer_info_t  timerParam = { .timeout = 5 };
#endif

/*******************************************************************************
* Function Name: ble_app_callback
****************************************************************************//**
*
* This is an event callback function to receive events from the BLE Component.
*
* \param event the event code.
*
* \param eventParam the event parameters.
*
* \return none.
*
*******************************************************************************/
static void ble_app_callback(uint32_t event, void* eventParam)
{
    cy_en_ble_api_result_t apiResult = CY_BLE_SUCCESS;
    /* Local variable to store the data received as part of the write request events */
    cy_stc_ble_gatts_write_cmd_req_param_t *writeReqParameter;
    //cy_stc_ble_gap_enhance_conn_complete_param_t *enhance_conn_param;

    switch (event)
    {
        /**********************************************************
        *                       General Events
        ***********************************************************/
        /* This event is received when the component is Started */
        case CY_BLE_EVT_STACK_ON:
            BLE_DBG_PRINTF("CY_BLE_EVT_STACK_ON, StartAdvertisement\r\n");
            /* Enter into discoverable mode so that remote can find it. */
            if(CY_BLE_SUCCESS != (apiResult = Cy_BLE_GAPP_StartAdvertisement(CY_BLE_ADVERTISING_FAST, \
                                                           CY_BLE_PERIPHERAL_CONFIGURATION_0_INDEX)))
            {
                BLE_DBG_PRINTF("Cy_BLE_GAPP_StartAdvertisement API Error: 0x%x\r\n", apiResult);
            }
            /* Generates the security keys */
            //if(CY_BLE_SUCCESS != (apiResult = Cy_BLE_GAP_GenerateKeys(&keyInfo)))
            //{
            //    BLE_DBG_PRINTF("Cy_BLE_GAP_GenerateKeys API Error: 0x%x\r\n", apiResult);
            //}
            /* Display Bond list */
            ble_display_bond_list();
            break;
            
        case CY_BLE_EVT_TIMEOUT:
            if(((cy_stc_ble_timeout_param_t *)eventParam)->reasonCode == CY_BLE_GENERIC_APP_TO)
            {
                #if ENABLE_FPMM_CMD_RESPONSE_ECHO == ENABLED
                if(((cy_stc_ble_timeout_param_t *)eventParam)->timerHandle == ble_res_timer_param.timerHandle)
                {
                    ble_res_timer_timeout_flag = true;
                }
                #endif
            }
            #if ENABLE_BLE_MAIN_TIMER == ENABLED
            if((((cy_stc_ble_timeout_param_t *)eventParam)->reasonCode == CY_BLE_GENERIC_APP_TO) && 
               (((cy_stc_ble_timeout_param_t *)eventParam)->timerHandle == timerParam.timerHandle))
            {
                mainTimer++;
                BLE_DBG_PRINTF("CY_BLE_EVT_TIMEOUT timer out\r\n");
            }
            #endif
            break;
            
        case CY_BLE_EVT_HARDWARE_ERROR:
            /* This event indicates that some internal HW error has occurred. */
            BLE_DBG_PRINTF("CY_BLE_EVT_HARDWARE_ERROR\r\n");
            /* Halt CPU in Debug mode */
            CY_ASSERT(0u != 0u);
            break;
            
        case CY_BLE_EVT_STACK_BUSY_STATUS:
            BLE_DBG_PRINTF("CY_BLE_EVT_STACK_BUSY_STATUS: %x\r\n", *(uint8_t *)eventParam);
            break;
        case CY_BLE_EVT_SET_TX_PWR_COMPLETE:
            BLE_DBG_PRINTF("CY_BLE_EVT_SET_TX_PWR_COMPLETE \r\n");
            break;
        case CY_BLE_EVT_LE_SET_EVENT_MASK_COMPLETE:
            BLE_DBG_PRINTF("CY_BLE_EVT_LE_SET_EVENT_MASK_COMPLETE \r\n");
            break;
            
        case CY_BLE_EVT_SET_DEVICE_ADDR_COMPLETE:
            BLE_DBG_PRINTF("CY_BLE_EVT_SET_DEVICE_ADDR_COMPLETE \r\n");
            /* Reads the BD device address from BLE Controller's memory */
            Cy_BLE_GAP_GetBdAddress();
            break;
        case CY_BLE_EVT_GET_DEVICE_ADDR_COMPLETE:
            BLE_DBG_PRINTF("CY_BLE_EVT_GET_DEVICE_ADDR_COMPLETE: ");
            for(uint8_t i = CY_BLE_GAP_BD_ADDR_SIZE; i > 0u; i--)
            {
                BLE_DBG_PRINTF("%2.2x", ((cy_stc_ble_bd_addrs_t *)\
                ((cy_stc_ble_events_param_generic_t *)eventParam)->eventParams)->publicBdAddr[i-1]);
            }
            BLE_DBG_PRINTF(", private:");
            for(uint8_t i = CY_BLE_GAP_BD_ADDR_SIZE; i > 0u; i--)
            {
                BLE_DBG_PRINTF("%2.2x", ((cy_stc_ble_bd_addrs_t *)((cy_stc_ble_events_param_generic_t *)eventParam)->
                                                                 eventParams)->privateBdAddr[i-1]);
            }
            BLE_DBG_PRINTF("\r\n");
            /* Generates the security keys */
            if(CY_BLE_SUCCESS != (apiResult = Cy_BLE_GAP_GenerateKeys(&keyInfo)))
            {
                BLE_DBG_PRINTF("Cy_BLE_GAP_GenerateKeys API Error: 0x%x\r\n", apiResult);
            }
            break;
            
        case CY_BLE_EVT_STACK_SHUTDOWN_COMPLETE:
            BLE_DBG_PRINTF("CY_BLE_EVT_STACK_SHUTDOWN_COMPLETE\r\n");
            break;
            
        /** This event notifies the Host of a change to either the maximum Payload length or
         *  the maximum transmission time of Data Channel PDUs in either direction. The values 
         *  reported are the maximum that will actually be used on the connection following the change.
         *  The event parameter is of type 'cy_stc_ble_data_length_change_event_param_t'
         */
        case CY_BLE_EVT_DATA_LENGTH_CHANGE:
            BLE_DBG_PRINTF("CY_BLE_EVT_DATA_LENGTH_CHANGE: connMaxTxOctets=%d, connMaxTxTime=%d, "
                "connMaxRxOctets=%d, connMaxRxTime=%d, bdHandle=0x%x\r\n", 
                (*(cy_stc_ble_data_length_change_event_param_t *)eventParam).connMaxTxOctets, 
                (*(cy_stc_ble_data_length_change_event_param_t *)eventParam).connMaxTxTime, 
                (*(cy_stc_ble_data_length_change_event_param_t *)eventParam).connMaxRxOctets, 
                (*(cy_stc_ble_data_length_change_event_param_t *)eventParam).connMaxRxTime, 
                (*(cy_stc_ble_data_length_change_event_param_t *)eventParam).bdHandle);
            break;
        case CY_BLE_EVT_SET_SUGGESTED_DATA_LENGTH_COMPLETE:
            BLE_DBG_PRINTF("CY_BLE_EVT_SET_SUGGESTED_DATA_LENGTH_COMPLETE\r\n");
            break;
            
        #if(CY_BLE_CONFIG_ENABLE_PHY_UPDATE != 0u)
        case CY_BLE_EVT_SET_DEFAULT_PHY_COMPLETE:
            BLE_DBG_PRINTF("CY_BLE_EVT_SET_DEFAULT_PHY_COMPLETE \r\n");
            break;
        #endif  /* (CY_BLE_CONFIG_ENABLE_PHY_UPDATE != 0u) */
        
        /**********************************************************
        *                       GAP Events
        ***********************************************************/
        case CY_BLE_EVT_GAP_KEYS_GEN_COMPLETE:
            BLE_DBG_PRINTF("CY_BLE_EVT_GAP_KEYS_GEN_COMPLETE \r\n");
            keyInfo.SecKeyParam = (*(cy_stc_ble_gap_sec_key_param_t *)eventParam);
            apiResult = Cy_BLE_GAP_SetIdAddress(&cy_ble_deviceAddress);
            if(apiResult != CY_BLE_SUCCESS)
            {
                BLE_DBG_PRINTF("Cy_BLE_GAP_SetIdAddress API Error: 0x%x \r\n", apiResult);
            }
            #if(CY_BLE_CONFIG_ENABLE_PHY_UPDATE != 0u)
            {
                const cy_stc_ble_set_suggested_phy_info_t phyInfo =
                {
                    .allPhyMask = CY_BLE_PHY_NO_PREF_MASK_NONE,
                    .txPhyMask = CY_BLE_PHY_MASK_LE_2M,
                    .rxPhyMask = CY_BLE_PHY_MASK_LE_2M
                };
                apiResult = Cy_BLE_SetDefaultPhy(&phyInfo);
                if(apiResult != CY_BLE_SUCCESS)
                {
                    BLE_DBG_PRINTF("Cy_BLE_SetDefaultPhy API Error: 0x%x \r\n", apiResult);
                }
            }
            #endif  /* (CY_BLE_CONFIG_ENABLE_PHY_UPDATE != 0u) */
            break;
            
        case CY_BLE_EVT_GAP_AUTH_REQ:
            BLE_DBG_PRINTF("CY_BLE_EVT_GAP_AUTH_REQ: bdHandle=%x, security=%x, bonding=%x, ekeySize=%x, err=%x \r\n", 
                (*(cy_stc_ble_gap_auth_info_t *)eventParam).bdHandle, (*(cy_stc_ble_gap_auth_info_t *)eventParam).security, 
                (*(cy_stc_ble_gap_auth_info_t *)eventParam).bonding, (*(cy_stc_ble_gap_auth_info_t *)eventParam).ekeySize, 
                (*(cy_stc_ble_gap_auth_info_t *)eventParam).authErr);
            if(cy_ble_configPtr->authInfo[CY_BLE_SECURITY_CONFIGURATION_0_INDEX].security == 
                (CY_BLE_GAP_SEC_MODE_1 | CY_BLE_GAP_SEC_LEVEL_1))
            {
                cy_ble_configPtr->authInfo[CY_BLE_SECURITY_CONFIGURATION_0_INDEX].authErr = 
                    CY_BLE_GAP_AUTH_ERROR_PAIRING_NOT_SUPPORTED;
            }
            cy_ble_configPtr->authInfo[CY_BLE_SECURITY_CONFIGURATION_0_INDEX].bdHandle = 
                ((cy_stc_ble_gap_auth_info_t *)eventParam)->bdHandle;
            /* Pass security information for authentication in reply to an authentication request 
             * from the master device */
            apiResult = Cy_BLE_GAPP_AuthReqReply(&cy_ble_configPtr->authInfo[CY_BLE_SECURITY_CONFIGURATION_0_INDEX]);
            if(apiResult != CY_BLE_SUCCESS)
            {
                if(CY_BLE_SUCCESS != (apiResult = Cy_BLE_GAP_RemoveOldestDeviceFromBondedList()))
                {
                    BLE_DBG_PRINTF("Cy_BLE_GAP_RemoveOldestDeviceFromBondedList API Error: 0x%x \r\n", apiResult);
                }
                else
                {
                    apiResult = Cy_BLE_GAPP_AuthReqReply(&cy_ble_configPtr->authInfo[CY_BLE_SECURITY_CONFIGURATION_0_INDEX]);
                    if(apiResult != CY_BLE_SUCCESS)
                    {
                        BLE_DBG_PRINTF("Cy_BLE_GAPP_AuthReqReply API Error: 0x%x \r\n", apiResult);
                    }
                }
            }
            break;
            
        case CY_BLE_EVT_GAP_PASSKEY_ENTRY_REQUEST:
            BLE_DBG_PRINTF("CY_BLE_EVT_GAP_PASSKEY_ENTRY_REQUEST\r\n");
            BLE_DBG_PRINTF("Please enter the passkey displayed on the peer device:\r\n");
            break;
            
        case CY_BLE_EVT_GAP_PASSKEY_DISPLAY_REQUEST:
            BLE_DBG_PRINTF("CY_BLE_EVT_GAP_PASSKEY_DISPLAY_REQUEST: %6.6ld\r\n", *(uint32_t *)eventParam);
            break;
            
        case CY_BLE_EVT_GAP_NUMERIC_COMPARISON_REQUEST:
            BLE_DBG_PRINTF("Compare this passkey with the one displayed in your peer device and press 'y' or 'n':"
                       " %6.6lu \r\n", *(uint32_t *)eventParam);
            break;
            
        case CY_BLE_EVT_GAP_KEYINFO_EXCHNGE_CMPLT:
            BLE_DBG_PRINTF("CY_BLE_EVT_GAP_KEYINFO_EXCHNGE_CMPLT \r\n");
            break;
            
        case CY_BLE_EVT_GAP_SMP_NEGOTIATED_AUTH_INFO:
            BLE_DBG_PRINTF("CY_BLE_EVT_GAP_SMP_NEGOTIATED_AUTH_INFO:"
                       " bdHandle=%x, security=%x, bonding=%x, ekeySize=%x, err=%x \r\n", 
                (*(cy_stc_ble_gap_auth_info_t *)eventParam).bdHandle, 
                (*(cy_stc_ble_gap_auth_info_t *)eventParam).security, 
                (*(cy_stc_ble_gap_auth_info_t *)eventParam).bonding, 
                (*(cy_stc_ble_gap_auth_info_t *)eventParam).ekeySize, 
                (*(cy_stc_ble_gap_auth_info_t *)eventParam).authErr);
            break;
            
        case CY_BLE_EVT_GAP_AUTH_COMPLETE:
            BLE_DBG_PRINTF("CY_BLE_EVT_GAP_AUTH_COMPLETE: bdHandle=%x, security=%x, bonding=%x, ekeySize=%x, err=%x \r\n", 
                (*(cy_stc_ble_gap_auth_info_t *)eventParam).bdHandle, 
                (*(cy_stc_ble_gap_auth_info_t *)eventParam).security, 
                (*(cy_stc_ble_gap_auth_info_t *)eventParam).bonding, 
                (*(cy_stc_ble_gap_auth_info_t *)eventParam).ekeySize, 
                (*(cy_stc_ble_gap_auth_info_t *)eventParam).authErr);
            break;
            
        case CY_BLE_EVT_GAP_AUTH_FAILED:
            BLE_DBG_PRINTF("CY_BLE_EVT_GAP_AUTH_FAILED, reason: ");
            switch((*(cy_stc_ble_gap_auth_info_t *)eventParam).authErr)
            {
                case CY_BLE_GAP_AUTH_ERROR_CONFIRM_VALUE_NOT_MATCH:
                    BLE_DBG_PRINTF("CONFIRM_VALUE_NOT_MATCH\r\n");
                    break;
                    
                case CY_BLE_GAP_AUTH_ERROR_INSUFFICIENT_ENCRYPTION_KEY_SIZE:
                    BLE_DBG_PRINTF("INSUFFICIENT_ENCRYPTION_KEY_SIZE\r\n");
                    break;
                
                case CY_BLE_GAP_AUTH_ERROR_UNSPECIFIED_REASON:
                    BLE_DBG_PRINTF("UNSPECIFIED_REASON\r\n");
                    break;
                    
                case CY_BLE_GAP_AUTH_ERROR_AUTHENTICATION_TIMEOUT:
                    BLE_DBG_PRINTF("AUTHENTICATION_TIMEOUT\r\n");
                    break;
                    
                default:
                    BLE_DBG_PRINTF("0x%x  \r\n", (*(cy_stc_ble_gap_auth_info_t *)eventParam).authErr);
                    break;
            }
            break;

            
        case CY_BLE_EVT_GAPP_ADVERTISEMENT_START_STOP:
            BLE_DBG_PRINTF("CY_BLE_EVT_GAPP_ADVERTISEMENT_START_STOP, state: %d \r\n", Cy_BLE_GetAdvertisementState());
            break;
            
    #if(CY_BLE_LL_PRIVACY_FEATURE_ENABLED)
        case CY_BLE_EVT_GAP_ENHANCE_CONN_COMPLETE:
        {
            cy_stc_ble_gap_enhance_conn_complete_param_t *enhConnParameters;
            uint8_t connectedBdHandle;
            enhConnParameters = (cy_stc_ble_gap_enhance_conn_complete_param_t *)eventParam;
            BLE_DBG_PRINTF("CY_BLE_EVT_GAP_ENHANCE_CONN_COMPLETE: %x, %x(%.2f ms), %x ", enhConnParameters->status,
                enhConnParameters->connIntv, (float)enhConnParameters->connIntv * CY_BLE_CONN_INTRV_TO_MS, enhConnParameters->connLatency);
            connectedBdHandle = enhConnParameters->bdHandle;
            if(enhConnParameters->status == 0x00)
            {
                BLE_DBG_PRINTF("\x1B[31m supervisionTO=%dms peerBdAddrType=%d peerBdAddr=",
                    enhConnParameters->supervisionTo*10, enhConnParameters->peerBdAddrType);
                for(uint8_t i = CY_BLE_GAP_BD_ADDR_SIZE; i > 0u; i--)
                {
                    BLE_DBG_PRINTF("%2.2x", enhConnParameters->peerBdAddr[i-1]);
                }
                if(enhConnParameters->peerBdAddrType == CY_BLE_GAP_RANDOM_RESOLVABLE_ADDR_TYPE)
                    BLE_DBG_PRINTF(" CY_BLE_GAP_RANDOM_RESOLVABLE_ADDR_TYPE");
                BLE_DBG_PRINTF("\x1B[0m");
            }
            BLE_DBG_PRINTF("\r\n");
    #else
        case CY_BLE_EVT_GAP_DEVICE_CONNECTED:
        {
            uint8_t connectedBdHandle;
            BLE_DBG_PRINTF("CY_BLE_EVT_GAP_DEVICE_CONNECTED: %x, %x(%.2f ms), %x, %x \r\n",   
                ((cy_stc_ble_gap_connected_param_t *)eventParam)->status,
                ((cy_stc_ble_gap_connected_param_t *)eventParam)->connIntv,
                (float)((cy_stc_ble_gap_connected_param_t *)eventParam)->connIntv * CY_BLE_CONN_INTRV_TO_MS,
                ((cy_stc_ble_gap_connected_param_t *)eventParam)->connLatency,
                ((cy_stc_ble_gap_connected_param_t *)eventParam)->supervisionTO);
            connectedBdHandle = ((cy_stc_ble_gap_connected_param_t *)eventParam)->bdHandle;
    #endif  /* CY_BLE_LL_PRIVACY_FEATURE_ENABLED */
            /* Initiate pairing process */
            if((cy_ble_configPtr->authInfo[CY_BLE_SECURITY_CONFIGURATION_0_INDEX].security & CY_BLE_GAP_SEC_LEVEL_MASK) > 
                CY_BLE_GAP_SEC_LEVEL_1)
            {
                cy_ble_configPtr->authInfo[CY_BLE_SECURITY_CONFIGURATION_0_INDEX].bdHandle = ble_app_conn_handle.bdHandle;
                apiResult = Cy_BLE_GAP_AuthReq(&cy_ble_configPtr->authInfo[CY_BLE_SECURITY_CONFIGURATION_0_INDEX]);
                if(apiResult != CY_BLE_SUCCESS)
                {
                    BLE_DBG_PRINTF(" Cy_BLE_GAP_AuthReq API Error: 0x%x \r\n", apiResult);
                }
            }
            /* Set security keys for new device which is not already bonded */
            if(false == ble_is_device_in_bond_list(connectedBdHandle))
            {
                keyInfo.SecKeyParam.bdHandle = connectedBdHandle;
                apiResult = Cy_BLE_GAP_SetSecurityKeys(&keyInfo);
                if(apiResult != CY_BLE_SUCCESS)
                {
                    BLE_DBG_PRINTF("Cy_BLE_GAP_SetSecurityKeys API Error: 0x%x \r\n", apiResult);
                }
            }
        }
            break;

        case CY_BLE_EVT_L2CAP_CONN_PARAM_UPDATE_RSP:
            BLE_DBG_PRINTF("CY_BLE_EVT_L2CAP_CONN_PARAM_UPDATE_RSP, result = %d\r\n", 
                (*(cy_stc_ble_l2cap_conn_update_rsp_param_t *)eventParam).result);
            break;
            
        case CY_BLE_EVT_GAP_CONNECTION_UPDATE_COMPLETE:
            /* in milliseconds / 1.25ms */
            BLE_DBG_PRINTF("CY_BLE_EVT_GAP_CONNECTION_UPDATE_COMPLETE: connIntv = %dms connLatency=%d supervisionTO=%dms\r\n",
                        ((cy_stc_ble_gap_conn_param_updated_in_controller_t *)eventParam)->connIntv * 5u /4u,
                        ((cy_stc_ble_gap_conn_param_updated_in_controller_t *)eventParam)->connLatency,
                        ((cy_stc_ble_gap_conn_param_updated_in_controller_t *)eventParam)->supervisionTO*10);
            if(6 < ((cy_stc_ble_gap_conn_param_updated_in_controller_t *)eventParam)->connIntv)
            {
                BLE_DBG_PRINTF("ble_app_connection_param_update_request\r\n");
                //ble_app_connection_param_update_request(6, 6, 0, 500);
            }
            break;
            
        case CY_BLE_EVT_GAP_DEVICE_DISCONNECTED:
            BLE_DBG_PRINTF("CY_BLE_EVT_GAP_DEVICE_DISCONNECTED: bdHandle=%x, reason=%x, status=%x\r\n",
                (*(cy_stc_ble_gap_disconnect_param_t *)eventParam).bdHandle, 
                (*(cy_stc_ble_gap_disconnect_param_t *)eventParam).reason, 
                (*(cy_stc_ble_gap_disconnect_param_t *)eventParam).status);
            /* Put the device into discoverable mode so that a remote can search it. */
            apiResult = Cy_BLE_GAPP_StartAdvertisement(CY_BLE_ADVERTISING_FAST, CY_BLE_PERIPHERAL_CONFIGURATION_0_INDEX);
            if(apiResult != CY_BLE_SUCCESS)
            {
                BLE_DBG_PRINTF("StartAdvertisement API Error: 0x%x \r\n", apiResult);
            }
            break;
            
        case CY_BLE_EVT_GAP_ENCRYPT_CHANGE:
            BLE_DBG_PRINTF("CY_BLE_EVT_GAP_ENCRYPT_CHANGE: %x \r\n", *(uint8_t *)eventParam);
            break;

        /**********************************************************
        *                       GATT Events
        ***********************************************************/
        /* This event is received when device is connected over GATT level */
        case CY_BLE_EVT_GATT_CONNECT_IND:
            ble_app_conn_handle = *(cy_stc_ble_conn_handle_t *)eventParam;
            BLE_DBG_PRINTF("CY_BLE_EVT_GATT_CONNECT_IND: %x, %x \r\n", 
                (*(cy_stc_ble_conn_handle_t *)eventParam).attId, (*(cy_stc_ble_conn_handle_t *)eventParam).bdHandle);
            break;
            
        /* This event is received when device is disconnected */
        case CY_BLE_EVT_GATT_DISCONNECT_IND:
            BLE_DBG_PRINTF("CY_BLE_EVT_GATT_DISCONNECT_IND: %x, %x \r\n", 
                (*(cy_stc_ble_conn_handle_t *)eventParam).attId, (*(cy_stc_ble_conn_handle_t *)eventParam).bdHandle);
            break;
            
        case CY_BLE_EVT_GATTS_XCNHG_MTU_REQ:
            {
                cy_stc_ble_gatt_xchg_mtu_param_t mtu = 
                {
                    .connHandle = ((cy_stc_ble_gatt_xchg_mtu_param_t *)eventParam)->connHandle
                };
                Cy_BLE_GATT_GetMtuSize(&mtu);
                BLE_DBG_PRINTF("CY_BLE_EVT_GATTS_XCNHG_MTU_REQ %x, %x, final mtu= %d \r\n", 
                    mtu.connHandle.attId, mtu.connHandle.bdHandle, mtu.mtu);
            }
            break;
            
        case CY_BLE_EVT_GATTS_READ_CHAR_VAL_ACCESS_REQ:
            BLE_DBG_PRINTF("CY_BLE_EVT_GATTS_READ_CHAR_VAL_ACCESS_REQ %x %x: handle: %x \r\n", 
                ((cy_stc_ble_gatts_char_val_read_req_t *)eventParam)->connHandle.attId,
                ((cy_stc_ble_gatts_char_val_read_req_t *)eventParam)->connHandle.bdHandle,
                ((cy_stc_ble_gatts_char_val_read_req_t *)eventParam)->attrHandle);
            break;

        case CY_BLE_EVT_GATTS_WRITE_REQ:
            BLE_DBG_PRINTF("CY_BLE_EVT_GATTS_WRITE_REQ\r\n");
            /* Read the write request parameter */
            writeReqParameter = (cy_stc_ble_gatts_write_cmd_req_param_t *)eventParam;
            /* Call the functions that handle the write request events for custom services */
            ble_fpmm_hi_write_request_handler(writeReqParameter);
            break;
            
        case CY_BLE_EVT_GATTS_WRITE_CMD_REQ:
            BLE_DBG_PRINTF("CY_BLE_EVT_GATTS_WRITE_CMD_REQ\r\n");
            /* Read the write request parameter */
            writeReqParameter = (cy_stc_ble_gatts_write_cmd_req_param_t *)eventParam;
            /* Call the functions that handle the write request events for custom services */
            ble_fpmm_hi_write_request_handler(writeReqParameter);
            break;
        
        case CY_BLE_EVT_GATTS_HANDLE_VALUE_CNF:
            BLE_DBG_PRINTF("CY_BLE_EVT_GATTS_HANDLE_VALUE_CNF: %x, %x \r\n", 
                (*(cy_stc_ble_conn_handle_t *)eventParam).attId, (*(cy_stc_ble_conn_handle_t *)eventParam).bdHandle);
            break;
        
        case CY_BLE_EVT_GATTS_INDICATION_DISABLED:
            BLE_DBG_PRINTF("CY_BLE_EVT_GATTS_INDICATION_DISABLED\r\n");
            break;
            
        case CY_BLE_EVT_GATTS_INDICATION_ENABLED:
            BLE_DBG_PRINTF("CY_BLE_EVT_GATTS_INDICATION_ENABLED\r\n");
            break;
        
        /**********************************************************
        *                       Other Events
        ***********************************************************/
        case CY_BLE_EVT_PENDING_FLASH_WRITE:
            /* Inform application that flash write is pending. Stack internal data 
            * structures are modified and require to be stored in Flash using 
            * Cy_BLE_StoreBondingData() */
            BLE_DBG_PRINTF("CY_BLE_EVT_PENDING_FLASH_WRITE\r\n");
            break;
            
        default:
            BLE_DBG_PRINTF("Other event: 0x%x\r\n", (unsigned int)event);
            break;
    }
}

/*******************************************************************************
* Function Name: fpmm_reponse_write_callback
****************************************************************************//**
*
* The callback function handler for BLE FPMM response write request.
*
* \param len received len.
*
* \param cmd point to the data buffer.
*
* \return none.
*
*******************************************************************************/
#if ENABLE_FPMM_CMD_RESPONSE_ECHO == ENABLED
static void fpmm_reponse_write_callback(uint32_t len, void *cmd)
{
    uint32_t i = 0;
    bool match_flag = true;
    
    if(FPMM_RES_ECHO_DATA_SIZE == len)
    {
        for(i=0; i<len; i++)
        {
            if(((uint8_t *)cmd)[i] != fpmm_res_echo_data[i])
            {
                match_flag = false;
            }
        }
        if(match_flag)
        {
            fpmm_res_sent_flag = true;
        }
    }
}
#endif

/*******************************************************************************
* Function Name: ble_app_init
****************************************************************************//**
*
* Initializes the BLE application.
*
* \param fpmm_cmd_wr_cb The FPMM command received callback.
*
* \return Return value indicates if the function succeeded or failed.
* see \ref cy_en_ble_api_result_t.
*
*******************************************************************************/
cy_en_ble_api_result_t ble_app_init(ble_fpmm_write_callback_t fpmm_cmd_wr_cb)
{
    cy_en_ble_api_result_t apiResult = CY_BLE_SUCCESS;
    ble_fpmm_hi_config_t fpmm_config;
    cy_stc_ble_stack_lib_version_t stackVersion;
    
    #if ENABLE_FPMM_CMD_RESPONSE_ECHO == ENABLED
    fpmm_res_sent_flag = false;
    #endif
    /* initialize the FPMM host interface */
    fpmm_config.cmd_callback_func = fpmm_cmd_wr_cb;
    #if ENABLE_FPMM_CMD_RESPONSE_ECHO == ENABLED
    fpmm_config.res_write_callback_func = fpmm_reponse_write_callback;
    #else
    fpmm_config.res_write_callback_func = NULL;
    #endif
    ble_fpmm_hi_init(&fpmm_config);
    /* Initialize BLE Services */
    ble_ias_init();
    
    /* Initialize Debug UART for BLE */
    BLE_UART_START();
    /* \x1b[2J\x1b[;H - ANSI ESC sequence for clear screen */
    BLE_DBG_PRINTF("\x1b[2J\x1b[;H");
    BLE_DBG_PRINTF("****************************************************************\r\n");
    BLE_DBG_PRINTF("*                             APP2                             *\r\n");
    BLE_DBG_PRINTF("****************************************************************\r\n");
    
    BLE_DBG_PRINTF("Test DFU M0\r\n");
    
    /* Start BLE component and register generic event handler */
    if(CY_BLE_SUCCESS != (apiResult = Cy_BLE_Start(ble_app_callback)))
    {
        BLE_DBG_PRINTF("Cy_BLE_Start API Error: ");
        ble_print_api_result(apiResult);
        return apiResult;
    }
    /* Output current stack version to UART */
    apiResult = Cy_BLE_GetStackLibraryVersion(&stackVersion);
    if(apiResult != CY_BLE_SUCCESS)
    {
        BLE_DBG_PRINTF("CyBle_GetStackLibraryVersion API Error: 0x%2.2x \r\n", apiResult);
    }
    else
    {
        BLE_DBG_PRINTF("Stack Version: %d.%d.%d.%d \r\n", stackVersion.majorVersion, 
            stackVersion.minorVersion, stackVersion.patch, stackVersion.buildNumber);
    }
    
    /* Initialize Battery Service */
    if(CY_BLE_SUCCESS != (apiResult = ble_bas_init()))
    {
        BLE_DBG_PRINTF("ble_bas_init API Error: ");
        ble_print_api_result(apiResult);
        return apiResult;
    }
    
    return apiResult;
}

/*******************************************************************************
* Function Name: ble_app_task
****************************************************************************//**
*
* BLE application task.
*
* \param none.
*
* \return Return value indicates if the function succeeded or failed.
* see \ref cy_en_ble_api_result_t.
*
*******************************************************************************/
cy_en_ble_api_result_t ble_app_task(void)
{
    cy_en_ble_api_result_t apiResult = CY_BLE_SUCCESS;
    
    /* Cy_BLE_ProcessEvents() allows BLE stack to process pending events */
    Cy_BLE_ProcessEvents();
    
    /* To achieve low power in the device */
    if(BLE_UART_DEB_IS_TX_COMPLETE() != 0u)
    {
        /* Entering into the Deep Sleep */
        Cy_SysPm_DeepSleep(CY_SYSPM_WAIT_FOR_INTERRUPT);
    }
    
    /* Restart timer */
    #if ENABLE_BLE_MAIN_TIMER == ENABLED
    if(mainTimer != 0u)
    {
        mainTimer = 0u;
        Cy_BLE_StartTimer(&timerParam);
    }
    #endif
    
    /* Restart the advertisement */
    if((Cy_BLE_GetState() == CY_BLE_STATE_ON) \
        && (Cy_BLE_GetAdvertisementState() == CY_BLE_ADV_STATE_STOPPED) \
        && (Cy_BLE_GetNumOfActiveConn() < 1))
    {
        apiResult = Cy_BLE_GAPP_StartAdvertisement(CY_BLE_ADVERTISING_FAST, CY_BLE_PERIPHERAL_CONFIGURATION_0_INDEX);
        if(apiResult != CY_BLE_SUCCESS)
        {
            BLE_DBG_PRINTF("Task StartAdvertisement API Error: ");
            ble_print_api_result(apiResult);
        }
        BLE_DBG_PRINTF("Task StartAdvertisement\r\n");
        Cy_BLE_ProcessEvents();
    }
    
    /* Update the battery level notification */
    if((Cy_BLE_GetConnectionState(ble_app_conn_handle) == CY_BLE_CONN_STATE_CONNECTED) \
        && (Cy_BLE_GetNumOfActiveConn() >= 1) \
        && (true == ble_battery_level_updated_flag))
    {
        ble_battery_level_updated_flag = false;
        apiResult = ble_bas_send_notification(ble_app_conn_handle, ble_battery_level);
        Cy_BLE_ProcessEvents();
    }
    
    /* Store bonding data to flash only when all debug information has been sent */
    #if(CY_BLE_BONDING_REQUIREMENT == CY_BLE_BONDING_YES)
    if(cy_ble_pendingFlashWrite != 0u) 
    {
        apiResult = Cy_BLE_StoreBondingData();
        BLE_DBG_PRINTF("Store bonding data, status: %x, pending: %x \r\n", apiResult, cy_ble_pendingFlashWrite);
    }
    #endif /* CY_BLE_BONDING_REQUIREMENT == CY_BLE_BONDING_YES */
    
    if (alertLevel != 0u)
    {
        /* Received alert to switch back to the bootloader */
        cy_stc_ble_gap_disconnect_info_t disconnectInfoParam =
        {
            .bdHandle = ble_app_conn_handle.bdHandle,
            .reason = CY_BLE_HCI_ERROR_OTHER_END_TERMINATED_USER
        };
        
        /* Initiate disconnection from the peer device*/
        if(Cy_BLE_GAP_Disconnect(&disconnectInfoParam) == CY_BLE_SUCCESS)
        {
            /* Wait for disconnection event */
            while(Cy_BLE_GetConnectionState(ble_app_conn_handle) == CY_BLE_CONN_STATE_CONNECTED)
            {
                /* Process BLE events */
                Cy_BLE_ProcessEvents();
            }
        }
        /* Stop BLE component. */
        Cy_BLE_Disable();
        Cy_Bootload_ExecuteApp(1u);
    }
    
    return CY_BLE_SUCCESS;
}

/*******************************************************************************
* Function Name: ble_app_send_fpmm_response
****************************************************************************//**
*
* Send the FPMM response data by ble.
*
* \param len The size of fpmmm response data..
*
* \param res Pont to the response data buffer.
*
* \return Return value indicates if the function succeeded or failed.
* see \ref cy_en_ble_api_result_t.
*
*******************************************************************************/
cy_en_ble_api_result_t ble_app_send_fpmm_response(uint32_t len, void *res)
{
    #if (BLE_DEBUG_UART_ENABLED == ENABLED)
    uint8 n = 0;
    BLE_DBG_PRINTF("FPMM RES: ");
    for(n=0; n<len; n++)
    {
        BLE_DBG_PRINTF("%02x ", (*((uint8_t *)res+n)));
    }
    BLE_DBG_PRINTF("\r\n");
    #endif
    cy_en_ble_api_result_t apiResult = CY_BLE_ERROR_INVALID_OPERATION;
    
    #if ENABLE_FPMM_CMD_RESPONSE_ECHO == ENABLED
    fpmm_res_sent_flag = false;
    #endif
    if(CY_BLE_SUCCESS == (apiResult = ble_fpmm_response_send_notification(ble_app_conn_handle, len, res)))
    {
        #if ENABLE_FPMM_CMD_RESPONSE_ECHO == ENABLED
        ble_res_timer_timeout_flag = false;
        if(CY_BLE_SUCCESS == Cy_BLE_StartTimer(&ble_res_timer_param))
        {
            while(true)
            {
                Cy_BLE_ProcessEvents();
                if(ble_res_timer_timeout_flag)
                {
                    apiResult = CY_BLE_ERROR_INVALID_OPERATION;
                    BLE_DBG_PRINTF("Didn't receive the response echo\r\n");
                    break;
                }
                if(fpmm_res_sent_flag)
                {
                    Cy_BLE_StopTimer(&ble_res_timer_param);
                    apiResult = CY_BLE_SUCCESS;
                    break;
                }
            } // while()...
        } // Cy_BLE_StartTimer....
        else
        {
            BLE_DBG_PRINTF("Cy_BLE_StartTimer API Error: 0x%x \r\n", apiResult);
        }
        #endif
    } // ble_fpmm_response_send_notification...
    else
    {
        BLE_DBG_PRINTF("ble_fpmm_response_send_notification API Error: 0x%x \r\n", apiResult);
    }
    return apiResult;
}

/*******************************************************************************
* Function Name: ble_app_stop
****************************************************************************//**
*
* Stops any ongoing operation in the BLE Stack and forces the BLE Stack to shut down.
*
* \param none
*
* \return Return value indicates if the function succeeded or failed.
* see \ref cy_en_ble_api_result_t.
*
*******************************************************************************/
cy_en_ble_api_result_t ble_app_stop(void)
{
    cy_en_ble_api_result_t apiResult = Cy_BLE_Stop();
    if(apiResult != CY_BLE_SUCCESS)
    {
        BLE_DBG_PRINTF("Cy_BLE_Stop API Error: ");
        ble_print_api_result(apiResult);
    }
    else
    {
        BLE_DBG_PRINTF("Wait for the BLE to Stop\r\n");
        while(CY_BLE_STATE_STOPPED != Cy_BLE_GetState())
        {
            Cy_BLE_ProcessEvents();
        }
        BLE_DBG_PRINTF("BLE Stopped!\r\n");
        BLE_UART_DEB_WAIT_TX_COMPLETE();
    }
    return apiResult;
}

/*******************************************************************************
* Function Name: ble_app_update_battery_level
****************************************************************************//**
*
* Update battery level to battery service.
*
* \param battery_level The value of battery level.
*
* \return Return value indicates if the function succeeded or failed.
* see \ref cy_en_ble_api_result_t.
*
*******************************************************************************/
cy_en_ble_api_result_t ble_app_update_battery_level(uint8_t battery_level)
{
    ble_battery_level = battery_level;
    ble_battery_level_updated_flag = true;
    return CY_BLE_SUCCESS;
}

/*******************************************************************************
* Function Name: ble_app_connection_param_update_request
****************************************************************************//**
*
* Request the peer Central device to update the connection parameters.
*
* \param interval_min  Minimum value for the connection event interval. This shall be less than
*                      or equal to conn_Interval_Max. Minimum connection interval will be
*                      connIntvMin * 1.25 ms. Time Range: 7.5 ms to 4 sec
*
* \param interval_max  Maximum value for the connection event interval. This shall be greater
*                      than or equal to conn_Interval_Min. Maximum connection interval will be
*                      connIntvMax * 1.25 ms. Time Range: 7.5 ms to 4 sec
*
* \param slave_latency Slave latency for the connection in number of connection events.
*                      Range: 0x0000 to 0x01F3
*
* \param timeout_multiplier Supervision timeout for the LE Link. Supervision timeout will be
*                           supervisionTO * 10 ms. Time Range: 100 msec to 32 secs
*
* \return Return value indicates if the function succeeded or failed.
* see \ref cy_en_ble_api_result_t.
*
*******************************************************************************/
cy_en_ble_api_result_t ble_app_connection_param_update_request(uint16_t interval_min, \
    uint16_t interval_max, uint16_t slave_latency, uint16_t timeout_multiplier)
{
    cy_en_ble_api_result_t apiResult = CY_BLE_SUCCESS;
    cy_stc_ble_gap_conn_update_param_info_t param;
    
    if(((6 > interval_min) || (interval_min > 3200)) || ((6 > interval_max) || (interval_max > 3200)) 
    || (500 < slave_latency) || ((10 > timeout_multiplier) || (timeout_multiplier > 3200)))
    {
        BLE_DBG_PRINTF("ble_app_connection_param_update_request Input param error\r\n");
        return CY_BLE_ERROR_INVALID_PARAMETER;
    }
    if((Cy_BLE_GetConnectionState(ble_app_conn_handle) != CY_BLE_CONN_STATE_CONNECTED) \
        && (Cy_BLE_GetNumOfActiveConn() < 1))
    {
        BLE_DBG_PRINTF("ble_app_connection_param_update_request No connection\r\n");
        return CY_BLE_ERROR_NO_CONNECTION;
    }
    param.bdHandle = ble_app_conn_handle.bdHandle;
    param.connIntvMin = interval_min;
    param.connIntvMax = interval_max;
    param.connLatency = slave_latency;
    param.supervisionTO = timeout_multiplier;
    if(CY_BLE_SUCCESS != (apiResult = Cy_BLE_L2CAP_LeConnectionParamUpdateRequest(&param)))
    {
        BLE_DBG_PRINTF("Cy_BLE_L2CAP_LeConnectionParamUpdateRequest API Error: 0x%x \r\n", apiResult);
        return apiResult;
    }
    return apiResult;
}

/* [] END OF FILE */
