/***************************************************************************//**
* \file ble_bas.c
* \version 1.0
*
* \brief
* Source file for BLE Battery Sevice.
*
********************************************************************************
* \copyright
* Copyright 2016-2018, Cypress Semiconductor Corporation. All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/
#include "ble_bas.h"


/*******************************************************************************
* Function Name: ble_bas_callback
****************************************************************************//**
*
* This is an event callback function to receive service-specific events 
* from Battery Service.
*
* \param event the event code.
*
* \param eventParam the event parameters.
*
* \return none.
*
*******************************************************************************/
static void ble_bas_callback(uint32_t event, void *eventParam)
{
    #if (BLE_DEBUG_UART_ENABLED == ENABLED)
    uint8_t locServiceIndex = ((cy_stc_ble_bas_char_value_t*)eventParam)->serviceIndex;
    BLE_DBG_PRINTF("BAS event: 0x%x, ", (unsigned int)event);
    #else
    eventParam = eventParam;
    #endif /* (BLE_DEBUG_UART_ENABLED == ENABLED) */
    
    switch(event)
    {
        /** Notification for Battery Level Characteristic was enabled.
         *  The parameter of this event is a structure of the cy_stc_ble_bas_char_value_t type.
         */
        case CY_BLE_EVT_BASS_NOTIFICATION_ENABLED:
            BLE_DBG_PRINTF("CY_BLE_EVT_BASS_NOTIFICATION_ENABLED %x %x: serviceIndex=%x\r\n",
                       ((cy_stc_ble_bas_char_value_t*)eventParam)->connHandle.attId,
                       ((cy_stc_ble_bas_char_value_t*)eventParam)->connHandle.bdHandle,
                       locServiceIndex);
            break;
        /** Notification for Battery Level Characteristic was disabled.
         *  The parameter of this event is a structure of the cy_stc_ble_bas_char_value_t type.
         */
        case CY_BLE_EVT_BASS_NOTIFICATION_DISABLED:
            BLE_DBG_PRINTF("CY_BLE_EVT_BASS_NOTIFICATION_DISABLED %x %x: serviceIndex=%x\r\n",
                       ((cy_stc_ble_bas_char_value_t*)eventParam)->connHandle.attId,
                       ((cy_stc_ble_bas_char_value_t*)eventParam)->connHandle.bdHandle,
                       locServiceIndex);
            break;
        case CY_BLE_EVT_BASC_NOTIFICATION:
            break;
        case CY_BLE_EVT_BASC_READ_CHAR_RESPONSE:
            break;
        case CY_BLE_EVT_BASC_READ_DESCR_RESPONSE:
            break;
        case CY_BLE_EVT_BASC_WRITE_DESCR_RESPONSE:
            break;
        default:
            BLE_DBG_PRINTF("Other event\r\n");
            break;
    }
}

/*******************************************************************************
* Function Name: ble_bas_init
****************************************************************************//**
*
* Initializes the battery service.
*
* \param none.
*
* \return Return value indicates if the function succeeded or failed.
* see \ref cy_en_ble_api_result_t.
*
*******************************************************************************/
cy_en_ble_api_result_t ble_bas_init(void)
{
    /* Register service specific callback function */
    return Cy_BLE_BAS_RegisterAttrCallback(ble_bas_callback);
}

/*******************************************************************************
* Function Name: ble_bas_send_notification
****************************************************************************//**
*
* Send the battery level notification to client.
*
* \param connHandle The ble connected handle.
*
* \param batteryLevel The battery level of BAS.
*
* \return Return value indicates if the function succeeded or failed.
* see \ref cy_en_ble_api_result_t.
*
*******************************************************************************/
cy_en_ble_api_result_t ble_bas_send_notification(cy_stc_ble_conn_handle_t connHandle, uint8_t batteryLevel)
{
    cy_en_ble_api_result_t apiResult = CY_BLE_SUCCESS;
    uint16_t cccd = 0;
    
    /* Get the CCCD of characteristic descriptor */
    apiResult = Cy_BLE_BASS_GetCharacteristicDescriptor(connHandle, CY_BLE_BAS_BATTERY_LEVEL, \
        CY_BLE_BAS_BATTERY_LEVEL, CY_BLE_BAS_BATTERY_LEVEL_CCCD, CY_BLE_CCCD_LEN, (uint8_t*)&cccd);
    if(CY_BLE_SUCCESS != apiResult)
    {
        BLE_DBG_PRINTF("Cy_BLE_BASS_GetCharacteristicDescriptor API Error: 0x%x \r\n", apiResult);
        return apiResult;
    }
    /* Notifications enabled */
    if(cccd == CY_BLE_CCCD_NOTIFICATION)
    {
        do
        {
            Cy_BLE_ProcessEvents();
        }
        while(Cy_BLE_GATT_GetBusyStatus(connHandle.attId) == CY_BLE_STACK_STATE_BUSY);
        /* Update Battery Level characteristic value and send Notification */
        apiResult = Cy_BLE_BASS_SendNotification(connHandle, CY_BLE_BAS_BATTERY_LEVEL, \
            CY_BLE_BAS_BATTERY_LEVEL, sizeof(batteryLevel), &batteryLevel);
        if(apiResult != CY_BLE_SUCCESS)
        {
            BLE_DBG_PRINTF("Cy_BLE_BASS_SendNotification API Error: 0x%x \r\n", apiResult);
            return apiResult;
        }
    }
    else
    {
        /* If notifications disabled,only update Battery Level characteristic value */
        apiResult = Cy_BLE_BASS_SetCharacteristicValue(CY_BLE_BAS_BATTERY_LEVEL, \
            CY_BLE_BAS_BATTERY_LEVEL, sizeof(batteryLevel), &batteryLevel);
        if(apiResult != CY_BLE_SUCCESS)
        {
            BLE_DBG_PRINTF("Cy_BLE_BASS_SetCharacteristicValue API Error: 0x%x \r\n", apiResult);
            return apiResult;
        }
    }
    return CY_BLE_SUCCESS;
}

/* [] END OF FILE */
