/***************************************************************************//**
* \file ble_fpmm_hi.c
* \version 1.0
*
* \brief
* Source file for BLE FPMM (Figerprint Match Module) host interface service.
*
********************************************************************************
* \copyright
* Copyright 2016-2018, Cypress Semiconductor Corporation. All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/
#include "ble_fpmm_hi.h"

/**
 * @brief Global Handle to internal BLE FPMM host interafce structure.
 */
static ble_fpmm_hi_config_t ble_fpmm_hi_config;


/*******************************************************************************
* Function Name: ble_fpmm_hi_init
****************************************************************************//**
*
* Initializes the FPMM host interface.
*
* \param config FPMM host interface configuration.
*
* \return Return value indicates if the function succeeded or failed.
* see \ref cy_en_ble_api_result_t.
*
*******************************************************************************/
cy_en_ble_api_result_t ble_fpmm_hi_init(ble_fpmm_hi_config_t *config)
{
    if(config == NULL)
    {
        return CY_BLE_ERROR_INVALID_PARAMETER;
    }
    /* FPMM command write callback */
    if(NULL != config->cmd_callback_func)
    {
        ble_fpmm_hi_config.cmd_callback_func = config->cmd_callback_func;
    }
    else
    {
        ble_fpmm_hi_config.cmd_callback_func = NULL;
    }
    /* FPMM response write callback */
    if(NULL != config->res_write_callback_func)
    {
        ble_fpmm_hi_config.res_write_callback_func = config->res_write_callback_func;
    }
    else
    {
        ble_fpmm_hi_config.res_write_callback_func = NULL;
    }
    ble_fpmm_hi_config.res_notification_status = CCCD_NOTIFY_DISABLED;
    return CY_BLE_SUCCESS;
}

/*******************************************************************************
* Function Name: ble_fpmm_command_write_request
****************************************************************************//**
*
* The FPMM command write request handler.
*
* \param writeRequest The data received as part of the write request events.
*
* \return Return value indicates if the function succeeded or failed.
* see \ref cy_en_ble_api_result_t.
*
*******************************************************************************/
static cy_en_ble_api_result_t ble_fpmm_command_write_request(cy_stc_ble_gatts_write_cmd_req_param_t *writeRequest)
{
    /* Check if the returned handle is matching to FPMM commad Write Attribute */
    if(FPMM_CMD_CHAR_HANDLE == writeRequest->handleValPair.attrHandle)
    {
        /* FPMM command callback */
        if((NULL != ble_fpmm_hi_config.cmd_callback_func) && (0 < writeRequest->handleValPair.value.len))
        {
            ble_fpmm_hi_config.cmd_callback_func(writeRequest->handleValPair.value.len, \
                                                 writeRequest->handleValPair.value.val);
            #if (BLE_DEBUG_UART_ENABLED == ENABLED)
            uint8_t n = 0;
            BLE_DBG_PRINTF("FPMM CMD: ");
            for(n=0; n<writeRequest->handleValPair.value.len; n++)
            {
                BLE_DBG_PRINTF("%02x ", (*((uint8_t *)(writeRequest->handleValPair.value.val)+n)));
            }
            BLE_DBG_PRINTF("\r\n");
            #endif
        }
    }
    return CY_BLE_SUCCESS;
}

/*******************************************************************************
* Function Name: ble_fpmm_response_write_request
****************************************************************************//**
*
* The FPMM response write request handler.
*
* \param writeRequest the data received as part of the write request events.
*
* \return Return value indicates if the function succeeded or failed.
* see \ref cy_en_ble_api_result_t.
*
*******************************************************************************/
static cy_en_ble_api_result_t ble_fpmm_response_write_request(cy_stc_ble_gatts_write_cmd_req_param_t *writeRequest, cy_en_ble_gatt_err_code_t *pgattErr)
{
    cy_en_ble_api_result_t apiResult = CY_BLE_SUCCESS;
    
    /* Check the validity and then extract the write value sent by the Client for FPMM Response CCCD */
    if(FPMM_RES_CCCD_HANDLE == writeRequest->handleValPair.attrHandle)
    {
        ble_fpmm_hi_config.res_notification_status = writeRequest->handleValPair.value.val[FPMM_RES_CCCD_INDEX];
        //BLE_DBG_PRINTF("res cccd value: 0x%x \r\n", ble_fpmm_hi_config.res_notification_status);
        cy_stc_ble_gatts_db_attr_val_info_t dbAttrValInfo =
        {
            .handleValuePair = writeRequest->handleValPair,
            .connHandle      = writeRequest->connHandle,
            .offset          = 0u,
            .flags           = CY_BLE_GATT_DB_PEER_INITIATED
        };
        *pgattErr = Cy_BLE_GATTS_WriteAttributeValueCCCD(&dbAttrValInfo);
        if(*pgattErr != CY_BLE_GATT_ERR_NONE)
        {
            apiResult = CY_BLE_ERROR_INVALID_STATE;
        }
    }
    /* Check if the returned handle is matching to FPMM response Write Attribute */
    else if(FPMM_RES_CHAR_HANDLE == writeRequest->handleValPair.attrHandle)
    {
        /* FPMM response write callback */
        if((NULL != ble_fpmm_hi_config.res_write_callback_func) && (0 < writeRequest->handleValPair.value.len))
        {
            ble_fpmm_hi_config.res_write_callback_func(writeRequest->handleValPair.value.len, \
                                                       writeRequest->handleValPair.value.val);
        }
    }
    return apiResult;
}

/*******************************************************************************
* Function Name: ble_fpmm_hi_write_request_handler
****************************************************************************//**
*
* The FPMM command and response write request handler.
*
* \param writeRequest The data received as part of the write request events.
*
* \return Return value indicates if the function succeeded or failed.
* see \ref cy_en_ble_api_result_t.
*
*******************************************************************************/
cy_en_ble_api_result_t ble_fpmm_hi_write_request_handler(cy_stc_ble_gatts_write_cmd_req_param_t *writeRequest)
{
    cy_en_ble_api_result_t apiResult1 = CY_BLE_SUCCESS;
    cy_en_ble_api_result_t apiResult2 = CY_BLE_SUCCESS;
    cy_en_ble_gatt_err_code_t gattErr = CY_BLE_GATT_ERR_NONE;
    
    if(writeRequest == NULL)
    {
        return CY_BLE_ERROR_INVALID_PARAMETER;
    }
    /* When this event is triggered, the peripheral has received a 
       write command on the custom  characteristic. Check if command
       fits any of the custom attributes and update the flag for
       sending notifications by the respective service */
    apiResult1 = ble_fpmm_response_write_request(writeRequest, &gattErr);
    apiResult2 = ble_fpmm_command_write_request(writeRequest);
    
    /* Send the response to the write request received. */
    cy_stc_ble_gatt_err_param_t err_param =
    {
        .errInfo.opCode     = CY_BLE_GATT_WRITE_REQ,
        .errInfo.attrHandle = writeRequest->handleValPair.attrHandle,
        .connHandle         = writeRequest->connHandle
    };
    /* Send response when event was handled by the service */
    if(gattErr != CY_BLE_GATT_ERR_NONE)
    {
        /* Send an Error Response */
        err_param.errInfo.errorCode  = gattErr;
        (void)Cy_BLE_GATTS_ErrorRsp(&err_param);
    }
    else
    {
        /* Send an Write Response */
        (void)Cy_BLE_GATTS_WriteRsp(writeRequest->connHandle);
    }
    
    if(apiResult1 != CY_BLE_SUCCESS)
    {
        return apiResult1;
    }
    else if(apiResult2 != CY_BLE_SUCCESS)
    {
        return apiResult2;
    }
    return CY_BLE_SUCCESS;
}

/*******************************************************************************
* Function Name: ble_fpmm_response_send_notification
****************************************************************************//**
*
* This function updates the notification status (lower byte of CCCD array) of
* a characteristic in GATT DB with the provided parameters.
*
* \param connHandle The ble connected handle.
*
* \param cccdHandle CCCD (Client Characteristic Configuration Descriptor) handle of the service.
*
* \param value Notification status. Valid values are CCCD_NOTIFY_DISABLED and CCCD_NOTIFY_ENABLED.
*
* \return Return value indicates if the function succeeded or failed.
* see \ref cy_en_ble_api_result_t.
*
*******************************************************************************/
cy_en_ble_api_result_t ble_fpmm_response_send_notification(cy_stc_ble_conn_handle_t connHandle, \
    uint32_t len, void *res)
{
    cy_en_ble_api_result_t apiResult = CY_BLE_SUCCESS;
    
    if(Cy_BLE_GetConnectionState(connHandle) != CY_BLE_CONN_STATE_CONNECTED)
    {
        return CY_BLE_ERROR_NO_CONNECTION;
    }
    
    if(CY_BLE_IS_NOTIFICATION_ENABLED(connHandle.attId, FPMM_RES_CCCD_HANDLE))
    {
        /* Wait for the stack is idle */
        while(Cy_BLE_GATT_GetBusyStatus(connHandle.attId) == CY_BLE_STACK_STATE_BUSY)
        {
            Cy_BLE_ProcessEvents();
        }
        /* Make sure that stack is not busy, then send the notification. Note that 
           the number of buffers in the BLE stack that holds the application data 
           payload are limited, and there are chances that notification might drop 
           a packet if the BLE stack buffers are not available. This error condition
           is not handled in this example project */
        if (Cy_BLE_GATT_GetBusyStatus(connHandle.attId) == CY_BLE_STACK_STATE_FREE)
        {
            /* Local variable that stores FPMM response notification data parameters */
            cy_stc_ble_gatts_handle_value_ntf_t fpmmResNotificationHandle =
            {
                .connHandle = connHandle,
                .handleValPair.attrHandle = FPMM_RES_CHAR_HANDLE,
                .handleValPair.value.len = len,
                .handleValPair.value.val = res,
            };
            /* Send the updated handle as part of attribute for notifications */
            apiResult = Cy_BLE_GATTS_Notification(&fpmmResNotificationHandle);
            if(apiResult != CY_BLE_SUCCESS)
            {
                BLE_DBG_PRINTF("Cy_BLE_GATTS_Notification API Error: 0x%x \r\n", apiResult);
            }
        }
        else
        {
            BLE_DBG_PRINTF("CY_BLE_STACK_STATE_BUSY\r\n");
        }
    }
    else
    {
        BLE_DBG_PRINTF("res_notification_status is not enabled \r\n");
        /* Store data in the database */
        cy_stc_ble_gatts_db_attr_val_info_t dbAttrValInfo =
        {
            .handleValuePair.attrHandle = FPMM_RES_CHAR_HANDLE,
            .handleValuePair.value.len  = len,
            .handleValuePair.value.val  = res,
            .offset = 0u,
            .flags = CY_BLE_GATT_DB_LOCALLY_INITIATED
        };
        if(Cy_BLE_GATTS_WriteAttributeValueCCCD(&dbAttrValInfo) != CY_BLE_GATT_ERR_NONE)
        {
            apiResult = CY_BLE_ERROR_INVALID_PARAMETER;
        }
    }
    /* Wait for the stack is idle */
    while(Cy_BLE_GATT_GetBusyStatus(connHandle.attId) == CY_BLE_STACK_STATE_BUSY)
    {
        Cy_BLE_ProcessEvents();
    }
    return apiResult;
}

/******************************************************************************
* Function Name: ble_fpmm_response_send_indication
***************************************************************************//**
*
*  Sends an indication with a characteristic value to the client's device.
*
*  \param connHandle:   The connection handle.
*  \param attrSize:     The size of the characteristic value attribute.
*  \param attrValue:    The pointer to the characteristic value data that should be
*                       sent to the client's device.
*  \param actualLen     Output parameter indicating actual length packed and sent over the air.
*
* \return
*  A return value of type \ref cy_en_ble_api_result_t.
*
*   Error Codes                              | Description
*   ------------                             | -----------
*   CY_BLE_SUCCESS                           | The request was handled successfully.
*   CY_BLE_ERROR_INVALID_PARAMETER           | Validation of the input parameter failed.
*   CY_BLE_ERROR_INVALID_OPERATION           | This operation is not permitted.
*   CY_BLE_ERROR_GATT_DB_INVALID_ATTR_HANDLE | An optional characteristic is absent.
*   CY_BLE_ERROR_INVALID_STATE               | Connection with the Client is not established.
*   CY_BLE_ERROR_MEMORY_ALLOCATION_FAILED    | Memory allocation failed.
*   CY_BLE_ERROR_IND_DISABLED                | Indication is not enabled by the Client.
******************************************************************************/
cy_en_ble_api_result_t ble_fpmm_response_send_indication(cy_stc_ble_conn_handle_t connHandle,
    uint16_t attrSize, void *attrValue, uint16_t *actualLen)
{
    /* Store new data in database */
    cy_en_ble_api_result_t apiResult;

    if((attrSize < 1) || (attrValue == NULL) || (actualLen == NULL))
    {
        return CY_BLE_ERROR_INVALID_PARAMETER;
    }
    /* Send indication if it is enabled and connected */
    if(Cy_BLE_GetConnectionState(connHandle) < CY_BLE_CONN_STATE_CONNECTED)
    {
        apiResult = CY_BLE_ERROR_INVALID_STATE;
    }
    else if(!CY_BLE_IS_INDICATION_ENABLED(connHandle.attId, FPMM_RES_CCCD_HANDLE))
    {
        apiResult = CY_BLE_ERROR_IND_DISABLED;
    }
    else
    {
        /* Get GATT MTU size */
        cy_stc_ble_gatt_xchg_mtu_param_t mtuParam = { .connHandle = connHandle };
        apiResult = Cy_BLE_GATT_GetMtuSize(&mtuParam);
        if(CY_BLE_SUCCESS != (apiResult = Cy_BLE_GATT_GetMtuSize(&mtuParam)))
        {
            return apiResult;
        }
        if((mtuParam.mtu - CY_BLE_GATT_WRITE_HEADER_LEN) < attrSize)
        {
            attrSize = mtuParam.mtu - CY_BLE_GATT_WRITE_HEADER_LEN;
        }
        
        /* Send the attribute value to to the peer device */
        #if 0
        /* Fill all fields of write request structure ... */
        cy_stc_ble_gatts_handle_value_ind_t indReqParam =
        {
            .handleValPair.attrHandle = FPMM_RES_CHAR_HANDLE,
            .handleValPair.value.val  = attrValue,
            .handleValPair.value.len  = attrSize,
            .connHandle               = connHandle
        };
        /* Send indication to client using previously filled structure */
        apiResult = Cy_BLE_GATTS_Indication(&indReqParam);
        #else
        cy_stc_ble_gatt_handle_value_pair_t indReqParam =
        {
            .attrHandle = FPMM_RES_CHAR_HANDLE,
            .value.val  = attrValue,
            .value.len  = attrSize
        };
        apiResult = Cy_BLE_GATTS_SendIndication(&connHandle, &indReqParam);
        #endif
        
        *actualLen = attrSize;
    }

    return(apiResult);
}



/* [] END OF FILE */
